local cvars = {
    ambience = 'Sound_AmbienceVolume',
    dialog = 'Sound_DialogVolume',
    master = 'Sound_MasterVolume',
    music = 'Sound_MusicVolume',
    sfx = 'Sound_SFXVolume',
}

local function setVolume(volume, channel)
    local channel = channel or 'master'
    local cvar = cvars[channel]
    if cvar then
        SetCVar(cvar, volume / 100)
        print('Set '..channel..' volume to '..volume..'%')
    else
        print('Unrecognized channel type: '..channel)
    end
end

local function toggleSoundInBackground()
    local setting = not (GetCVar('Sound_EnableSoundWhenGameIsInBG') == '1')
    SetCVar('Sound_EnableSoundWhenGameIsInBG', setting)
    print('Set sound in background to '..(setting and 'enabled' or 'disabled'))
end

local function usage()
    print('QuickVol usage: /quickvol <command> or /qv <command>')
    print('Commands:')
    print('/quickvol <number 0-100> [ambience|dialog|master|music|sfx]')
    print('    Set volume on specified channel (defaults to master if omitted).')
    print('/quickvol reset')
    print('    Restarts the game\'s sound system.')
    print('/quickvol bg')
    print('    Toggles sound in background setting.')
	print('Other commands are /quickvol raid|r, /quickvol record /quickvol normal')
	print('    These 3 are presets. Raid will mute certain sounds. Record will enable the standard ones. Normal is the same as record but with 20% volume.')
end

---------------------------------------

SLASH_QUICKVOL1 = '/quickvol'
SLASH_QUICKVOL2 = '/qv'

function SlashCmdList.QUICKVOL(msg, editBox)
    local parts = {strsplit(' ', msg)}
    if tonumber(parts[1]) then
        setVolume(tonumber(parts[1]), parts[2])
    elseif parts[1] == 'reset' then
        Sound_GameSystem_RestartSoundSystem()
    elseif parts[1] == 'record' then
		setVolume(100)
		SetCVar('Sound_EnableSFX', 1)
		SetCVar('Sound_EnableMusic', 1)
		SetCVar('Sound_EnableAmbience', 1)
		SetCVar('Sound_EnableDialog', 1)
        Sound_GameSystem_RestartSoundSystem()
    elseif parts[1] == 'raid' then
		setVolume(100)
		SetCVar('Sound_EnableSFX', 0)
		SetCVar('Sound_EnableMusic', 0)
		SetCVar('Sound_EnableAmbience', 0)
		SetCVar('Sound_EnableDialog', 0)
        Sound_GameSystem_RestartSoundSystem()
    elseif parts[1] == 'normal' then
		setVolume(20)
		SetCVar('Sound_EnableSFX', 1)
		SetCVar('Sound_EnableMusic', 1)
		SetCVar('Sound_EnableAmbience', 1)
		SetCVar('Sound_EnableDialog', 1)
        Sound_GameSystem_RestartSoundSystem()
    elseif parts[1] == 'bg' then
        toggleSoundInBackground()
    else
        usage()
    end
end
